@@ -34,6 +34,8 @@ STRING = Schema(
 )
 STRING_OR_NONE = Schema(Or(STRING, None))
 
+BOOL_OR_NONE = Schema(Or(bool, None))
+
 AABB = Schema(Or([[Use(float), Use(float)], [Use(float), Use(float)]], None))
 
 
@@ -366,7 +368,17 @@ INVENTORY_FILTER = Schema(
     )
 )
 
-REQUEST_FILTERS = Schema([(STRING, int)])  # TODO: change this
+
+def normalize_request(filters):
+    for i, entry in enumerate(filters):
+        if isinstance(entry, tuple):
+            filters[i] = {"index": i + 1, "name": entry[0], "count": entry[1]}
+    return filters
+
+
+REQUEST_FILTERS = Schema(
+    And(normalize_request, [{"index": int, "name": STRING, "count": int}])
+)  # TODO: change this
 
 WAIT_CONDITION = Schema(
     {
